unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, SQLDB, DB, SQLite3Conn, Forms, Controls, Graphics, Dialogs,
  DBGrids, StdCtrls, ComCtrls,
  bass;

type
  
  { TForm1 }

  TForm1 = class(TForm)
    btnAdd: TButton;
    btnPlay: TButton;
    btnStop: TButton;
    btnResume: TButton;
    btnPause: TButton;
    DataSource1: TDataSource;
    DBGrid1: TDBGrid;
    OpenDialog1: TOpenDialog;
    SQLite3Connection1: TSQLite3Connection;
    SQLQuery1: TSQLQuery;
    SQLTransaction1: TSQLTransaction;
    TrackBar1: TTrackBar;
    procedure btnAddClick(Sender: TObject);
    procedure btnPauseClick(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure btnResumeClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
  private
    FChannel: DWord;
    FMem: Pointer;
    function GetVolume: Double;
    procedure SetVolume(AValue: Double);

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

const
  FILE_NAME = 'data.db';
  
{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  H: THandle;
begin
  H := Handle;
  BASS_Init(-1, 44100, 0, {$IFNDEF MSWINDOWS}@{$ENDIF}H, nil);

  SQLite3Connection1.DatabaseName := FILE_NAME;
  if not FileExists(FILE_NAME) then
  begin
    SQLite3Connection1.Connected := true;
    SQLTransaction1.StartTransaction;
    SQLite3Connection1.ExecuteDirect(
      'CREATE TABLE Music (' +
        'ID INTEGER PRIMARY KEY AUTOINCREMENT, '+
        'SongName VARCHAR(40), '+
        'Song BLOB);'
      ); 
    SQLTransaction1.Commit;
    SQLTransaction1.EndTransaction;
  end;

  SQLQuery1.Options := [sqoAutoApplyUpdates, sqoAutoCommit, sqoKeepOpenOnCommit];
  SQLQuery1.SQL.Text := 'SELECT * FROM Music;';
  SQLite3Connection1.Connected := true;
  SQLTransaction1.StartTransaction;
  SQLQuery1.Open;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  btnStopClick(nil);
end;

function TForm1.GetVolume: Double;
begin
  Result := (TrackBar1.Position - TrackBar1.Min) / (TrackBar1.Max - TrackBar1.Min);
end;

procedure TForm1.SetVolume(AValue: Double);
begin
  Trackbar1.Position := Trackbar1.Min + round(AValue*(TrackBar1.Max - TrackBar1.Min));
  Trackbar1Change(nil);
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
  BASS_ChannelSetAttribute(FChannel, BASS_ATTRIB_VOL, GetVolume);
end;

procedure TForm1.btnAddClick(Sender: TObject);
begin
  if OpenDialog1.Execute then begin
    SQLQuery1.Append;
    SQLQuery1.FieldByName('SongName').AsString := ChangeFileExt(ExtractFileName(OpenDialog1.FileName), '');
    TBlobField(SQLQuery1.FieldByName('Song')).LoadFromFile(OpenDialog1.FileName);
    SQLQuery1.Post;
  end;
end;

procedure TForm1.btnPauseClick(Sender: TObject);
begin
  BASS_ChannelPause(FChannel);
end;

procedure TForm1.btnPlayClick(Sender: TObject);
const
  FROM_START = 0; 
var
  stream: TMemoryStream;
  flags: DWord = 0;
begin
  stream := TMemoryStream.Create;
  try
    TBlobField(SQLQuery1.FieldByName('Song')).SaveToStream(stream);
    stream.Position := 0;
    if FMem <> nil then
    begin
      FreeMem(FMem);
      FMem := nil;
    end;
    GetMem(FMem, stream.Size);
    stream.Read(FMem^, stream.Size);
    
    FChannel := BASS_StreamCreateFile(
      true,              // play from memory
      FMem,              // Pointer to memory location
      FROM_START,        // play file from start
      stream.Size,       // ... to end
      flags
    ); 
    SetVolume(GetVolume);
    BASS_ChannelPlay(FChannel, true);  // true = begin from start
  finally
    stream.Free;
  end;
end;

procedure TForm1.btnResumeClick(Sender: TObject);
begin
  BASS_ChannelPlay(FChannel, false);
end;

procedure TForm1.btnStopClick(Sender: TObject);
begin
  BASS_ChannelStop(FChannel);
  if FMem <> nil then
  begin
    FreeMem(FMem);
    FMem := nil;
  end;
end;

end.

